/*
 * rng.hpp
 *
 * Copyright (c) Stephen Thompson, 2007.
 * Licenced for non-commercial use only. See LICENCE.txt for details.
 *
 * Random number generators.
 *
 * getInt, getFloat return number in the range [a,b).
 * (TODO) will probably want to replace this with something using boost.
 * 
 */

#ifndef RNG_HPP
#define RNG_HPP

class RNG {
public:
	virtual ~RNG() { }

	virtual void setSeed(unsigned int=0) = 0;	// 0 means "use time"
	virtual float getU01() = 0;

	bool getBool(float p = 0.5f) { return getU01() <= p; } // (TODO) a version using ints from 0 to 100 instead?
	int getInt(int a, int b) { return a + int(getU01() * (b-a)); }
	float getFloat(float a, float b) { return a + getU01() * (b-a); }
};

// for std::random_shuffle compatibility
class RNG_Wrapper {
public:
	RNG_Wrapper(RNG &r_) : r(r_) { }
	int operator()(int n) { return r.getInt(0, n); }
private:
	RNG &r;
};

// concrete RNG
class StandardRNG : public RNG {
public:
	// Uses srand and rand. Note that this is not really reentrant.
	StandardRNG(unsigned int seed=0) { setSeed(0); }
	virtual void setSeed(unsigned int=0);
	virtual float getU01();
};

// global access point
extern StandardRNG g_rng;

#endif		// RNG_HPP
